#include "xmlparsingmodel.h"
#include <QFileInfo>
#include <QFont>

XMLParsingModel::XMLParsingModel(const QStringList& filepaths, QObject* parent)
    : QAbstractItemModel(parent)
{
    foreach(const auto& path, filepaths){
        QFile file(path, this);
        QFileInfo file_info(file.fileName());
        QString filename = file_info.fileName();

        if(!filenames.contains(filename)){
            if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
                qDebug() << "Fail to open";
            else{
                QDomDocument m_doc;
                if (!m_doc.setContent(&file))
                    qDebug() << "Fail to parse";
                else{
                    paths.append(path);
                    filenames.append(filename);
                    opened_docs.append(m_doc);

                    set_tag_name(filename);
                }
            }
        }
        file.close();
    }

    auto root_node = QDomDocument().createElement("root");

    foreach(const auto& d, opened_docs){
        root_node.appendChild(d.documentElement());
    }

    root_item = new XMLParsingItem(root_node, 0);
}

void XMLParsingModel::add_file(const QStringList &filepaths)
{
    foreach(const auto& path, filepaths){
        QFile file(path, this);
        QFileInfo file_info(file.fileName());
        QString filename = file_info.fileName();

        if(!filenames.contains(filename)){
            if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
                qDebug() << "Fail to open";
            else{
                QDomDocument m_doc;
                if (!m_doc.setContent(&file))
                    qDebug() << "Fail to parse";
                else{
                    paths.append(path);
                    filenames.append(filename);
                    opened_docs.append(m_doc);

                    set_tag_name(filename);
                }
            }
        }
    }

    foreach(const auto& d, opened_docs){
        root_item->node().appendChild(d.documentElement());
    }

}

void XMLParsingModel::delete_file(const QString &filename)
{
    if(filenames.contains(filename)){
        qsizetype ind = filenames.indexOf(filename);
        root_item->node().removeChild(root_item->node().childNodes().at(ind));
        paths.removeAt(ind);
        filenames.removeAt(ind);
        opened_docs.removeAt(ind);
        root_item->remove_child(ind);
    }
}

void XMLParsingModel::make_bold(const QModelIndex &index)
{
    bold_indexes[index.sibling(index.row(), 0).data().toString()] = index;
}

void XMLParsingModel::make_not_bold(const QString &filename)
{
    bold_indexes.remove(filename);
}

const QString &XMLParsingModel::get_path(const QString& filename)
{
    return paths[filenames.indexOf(filename)];
}


XMLParsingModel::~XMLParsingModel()
{
    delete root_item;
}

int XMLParsingModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 3;
}

XMLParsingItem *XMLParsingModel::get_root_item() const
{
    return root_item;
}

Qt::ItemFlags XMLParsingModel::flags(const QModelIndex& p_idx) const
{
    if (!p_idx.isValid())
        return Qt::NoItemFlags;
    return QAbstractItemModel::flags(p_idx);
}

QVariant XMLParsingModel::headerData(int section, Qt::Orientation p_orient, int role) const
{
    if (p_orient == Qt::Horizontal && role == Qt::DisplayRole) {
        switch (section) {
            case 0:
                return tr("Name");
            case 1:
                return tr("Attributes");
            case 2:
                return tr("Value");
            default:
                break;
        }
    }
    return QVariant();
}

QModelIndex XMLParsingModel::index(int row, int column, const QModelIndex &parent) const
{
    if(!hasIndex(row, column, parent))
        return QModelIndex();

    XMLParsingItem* parent_item;

    if(!parent.isValid())
        parent_item = root_item;
    else
        parent_item = static_cast<XMLParsingItem*>(parent.internalPointer());

    XMLParsingItem* child_item = parent_item->child(row);
    if(child_item)
        return createIndex(row, column, child_item);
    return QModelIndex();
}

int XMLParsingModel::rowCount(const QModelIndex &parent) const
{
    if (parent.column() > 0)
        return 0;

    XMLParsingItem* parentItem;

    if(!parent.isValid())
        parentItem = root_item;
    else
        parentItem = static_cast<XMLParsingItem*>(parent.internalPointer());
    return parentItem->node().childNodes().count();
}

QModelIndex XMLParsingModel::parent(const QModelIndex &child) const
{
    if (!child.isValid())
        return QModelIndex();

    XMLParsingItem *childItem = static_cast<XMLParsingItem*>(child.internalPointer());
    XMLParsingItem *parentItem = childItem->parent();

    if (!parentItem || parentItem == root_item)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);
}

QVariant XMLParsingModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if(role==Qt::FontRole && bold_indexes.values().contains(index))
        return QFont("Calibri", 10, QFont::Bold);

    if (role != Qt::DisplayRole)
        return QVariant();

    const XMLParsingItem *item = static_cast<XMLParsingItem*>(index.internalPointer());

    const QDomNode node = item->node();

    switch (index.column()) {
        case 0:
            return node.nodeName();
        case 1:
        {
            const QDomNamedNodeMap attribute_map = node.attributes();
            QStringList attributes;
            for (int i = 0; i < attribute_map.count(); ++i) {
                QDomNode attribute = attribute_map.item(i);

                attributes << attribute.nodeName() + "=\""
                              + attribute.nodeValue() + '"';
            }
            return attributes.join(' ');
        }
        case 2:
            return node.nodeValue();
        default:
            break;
    }
    return QVariant();
}

void XMLParsingModel::set_tag_name(const QString& filename)
{
    QDomElement catalog_node = opened_docs[filenames.indexOf(filename)].firstChildElement("catalog");
    catalog_node.setTagName(filename);
}

QDomNode XMLParsingModel::find_node(const QString &filename)
{
    auto node = root_item->node().firstChild();
    while (!node.isNull()) {
        if(node.toElement().tagName()==filename)
            return node;
        node = node.nextSibling();
    }
    return node;
}

XMLParsingItem::XMLParsingItem(const QDomNode& p_node, int row, XMLParsingItem* parent)
    : m_node(p_node)
    , parent_item(parent)
    , row_number(row)
{}

XMLParsingItem::~XMLParsingItem()
{
    qDeleteAll(child_items);
}

XMLParsingItem* XMLParsingItem::parent()
{
    return parent_item;
}

int XMLParsingItem::row() const
{
    return row_number;
}

QDomNode XMLParsingItem::node() const
{
    return m_node;
}


XMLParsingItem* XMLParsingItem::child(int i)
{
    XMLParsingItem* child_item = child_items.value(i);
    if(child_item)
        return child_item;
    if (i >= 0 && i < m_node.childNodes().count()){
        QDomNode child_node = m_node.childNodes().item(i); //QDomNode::childNodes.item(i) -> QDomNodeList[i]
        child_item = new XMLParsingItem(child_node, i, this);
        child_items.push_back(child_item);
    }
    return child_item;
}

void XMLParsingItem::remove_child(int i)
{
    child_items.removeAt(i);
    foreach(const auto& child, child_items){
        if(child->row()>i)
            --child->row_number;
    }
}

